@@ -16,7 +16,7 @@ or "icmp" as values for the --protocol parameter.  The actual Protocol
 parameter of the operation accepts only integer protocol numbers.
 """
 
-def _fix_args(operation, endpoint, params, **kwargs):
+def _fix_args(params, **kwargs):
     key_name = 'Protocol'
     if key_name in params:
         if params[key_name] == 'tcp':
