@@ -1,4 +1,5 @@
 import os
+import logging
 import sys
 import time
 from functools import partial
@@ -10,7 +11,11 @@ from dateutil.tz import tzlocal
 
 from botocore.compat import quote
 from awscli.customizations.s3.utils import find_bucket_key, \
-    uni_print, guess_content_type, MD5Error, bytes_print
+    uni_print, guess_content_type, MD5Error, bytes_print, set_file_utime, \
+    RequestParamsMapper
+
+
+LOGGER = logging.getLogger(__name__)
 
 
 class CreateDirectoryError(Exception):
@@ -34,7 +39,6 @@ def save_file(filename, response_data, last_update, is_stream=False):
     """
     body = response_data['Body']
     etag = response_data['ETag'][1:-1]
-    sse = response_data.get('ServerSideEncryption', None)
     if not is_stream:
         d = os.path.dirname(filename)
         try:
@@ -55,7 +59,7 @@ def save_file(filename, response_data, last_update, is_stream=False):
         with open(filename, 'wb') as out_file:
             write_to_file(out_file, etag, md5, file_chunks)
 
-    if not _is_multipart_etag(etag) and sse != 'aws:kms':
+    if _can_validate_md5_with_etag(etag, response_data):
         if etag != md5.hexdigest():
             if not is_stream:
                 os.remove(filename)
@@ -64,13 +68,22 @@ def save_file(filename, response_data, last_update, is_stream=False):
     if not is_stream:
         last_update_tuple = last_update.timetuple()
         mod_timestamp = time.mktime(last_update_tuple)
-        os.utime(filename, (int(mod_timestamp), int(mod_timestamp)))
+        set_file_utime(filename, int(mod_timestamp))
     else:
         # Now write the output to stdout since the md5 is correct.
         bytes_print(payload)
         sys.stdout.flush()
 
 
+def _can_validate_md5_with_etag(etag, response_data):
+    sse = response_data.get('ServerSideEncryption', None)
+    sse_customer_algorithm = response_data.get('SSECustomerAlgorithm', None)
+    if not _is_multipart_etag(etag) and sse != 'aws:kms' and \
+            sse_customer_algorithm is None:
+        return True
+    return False
+
+
 def write_to_file(out_file, etag, md5, file_chunks, is_stream=False):
     """
     Updates the etag for each file chunk.  It will write to the file if it a
@@ -137,6 +150,11 @@ class TaskInfo(object):
         bucket, key = find_bucket_key(self.src)
         self.client.delete_bucket(Bucket=bucket)
 
+    def is_glacier_compatible(self):
+        # These operations do not involving transferring glacier objects
+        # so they are always glacier compatible.
+        return True
+
 
 class FileInfo(TaskInfo):
     """
@@ -160,11 +178,16 @@ class FileInfo(TaskInfo):
     :param dest_type: string
     :param parameters: a dictionary of important values this is assigned in
         the ``BasicTask`` object.
+    :param associated_response_data: The response data used by
+        the ``FileGenerator`` to create this task. It is either an dictionary
+        from the list of a ListObjects or the response from a HeadObject. It
+        will only be filled if the task was generated from an S3 bucket.
     """
     def __init__(self, src, dest=None, compare_key=None, size=None,
                  last_update=None, src_type=None, dest_type=None,
                  operation_name=None, client=None, parameters=None,
-                 source_client=None, is_stream=False):
+                 source_client=None, is_stream=False,
+                 associated_response_data=None):
         super(FileInfo, self).__init__(src, src_type=src_type,
                                        operation_name=operation_name,
                                        client=client)
@@ -174,13 +197,12 @@ class FileInfo(TaskInfo):
         self.size = size
         self.last_update = last_update
         # Usually inject ``parameters`` from ``BasicTask`` class.
+        self.parameters = {}
         if parameters is not None:
             self.parameters = parameters
-        else:
-            self.parameters = {'acl': None,
-                               'sse': None}
         self.source_client = source_client
         self.is_stream = is_stream
+        self.associated_response_data = associated_response_data
 
     def set_size_from_s3(self):
         """
@@ -189,59 +211,41 @@ class FileInfo(TaskInfo):
         bucket, key = find_bucket_key(self.src)
         params = {'Bucket': bucket,
                   'Key': key}
+        RequestParamsMapper.map_head_object_params(params, self.parameters)
         response_data = self.client.head_object(**params)
         self.size = int(response_data['ContentLength'])
 
-    def _permission_to_param(self, permission):
-        if permission == 'read':
-            return 'GrantRead'
-        if permission == 'full':
-            return 'GrantFullControl'
-        if permission == 'readacl':
-            return 'GrantReadACP'
-        if permission == 'writeacl':
-            return 'GrantWriteACP'
-        raise ValueError('permission must be one of: '
-                         'read|readacl|writeacl|full')
+    def is_glacier_compatible(self):
+        """Determines if a file info object is glacier compatible
 
-    def _handle_object_params(self, params):
-        if self.parameters['acl']:
-            params['ACL'] = self.parameters['acl'][0]
-        if self.parameters['grants']:
-            for grant in self.parameters['grants']:
-                try:
-                    permission, grantee = grant.split('=', 1)
-                except ValueError:
-                    raise ValueError('grants should be of the form '
-                                     'permission=principal')
-                params[self._permission_to_param(permission)] = grantee
-        if self.parameters['sse']:
-            params['ServerSideEncryption'] = 'AES256'
-        if self.parameters['storage_class']:
-            params['StorageClass'] = self.parameters['storage_class'][0]
-        if self.parameters['website_redirect']:
-            params['WebsiteRedirectLocation'] = \
-                    self.parameters['website_redirect'][0]
-        if self.parameters['guess_mime_type']:
-            self._inject_content_type(params, self.src)
-        if self.parameters['content_type']:
-            params['ContentType'] = self.parameters['content_type'][0]
-        if self.parameters['cache_control']:
-            params['CacheControl'] = self.parameters['cache_control'][0]
-        if self.parameters['content_disposition']:
-            params['ContentDisposition'] = \
-                    self.parameters['content_disposition'][0]
-        if self.parameters['content_encoding']:
-            params['ContentEncoding'] = self.parameters['content_encoding'][0]
-        if self.parameters['content_language']:
-            params['ContentLanguage'] = self.parameters['content_language'][0]
-        if self.parameters['expires']:
-            params['Expires'] = self.parameters['expires'][0]
+        Operations will fail if the S3 object has a storage class of GLACIER
+        and it involves copying from S3 to S3, downloading from S3, or moving
+        where S3 is the source (the delete will actually succeed, but we do
+        not want fail to transfer the file and then successfully delete it).
 
-    def _handle_metadata_directive(self, params):
-        if self.parameters['metadata_directive']:
-            params['MetadataDirective'] = \
-                self.parameters['metadata_directive'][0]
+        :returns: True if the FileInfo's operation will not fail because the
+            operation is on a glacier object. False if it will fail.
+        """
+        if self._is_glacier_object(self.associated_response_data):
+            if self.operation_name in ['copy', 'download']:
+                return False
+            elif self.operation_name == 'move':
+                if self.src_type == 's3':
+                    return False
+        return True
+
+    def _is_glacier_object(self, response_data):
+        if response_data:
+            if response_data.get('StorageClass') == 'GLACIER' and \
+                    not self._is_restored(response_data):
+                return True
+        return False
+
+    def _is_restored(self, response_data):
+        # Returns True is this is a glacier object that has been
+        # restored back to S3.
+        # 'Restore' looks like: 'ongoing-request="false", expiry-date="..."'
+        return 'ongoing-request="false"' in response_data.get('Restore', '')
 
     def upload(self, payload=None):
         """
@@ -261,14 +265,32 @@ class FileInfo(TaskInfo):
             'Key': key,
             'Body': body,
         }
-        self._handle_object_params(params)
+        self._inject_content_type(params)
+        RequestParamsMapper.map_put_object_params(params, self.parameters)
         response_data = self.client.put_object(**params)
 
-    def _inject_content_type(self, params, filename):
+    def _inject_content_type(self, params):
+        if not self.parameters['guess_mime_type']:
+            return
+        filename = self.src
         # Add a content type param if we can guess the type.
-        guessed_type = guess_content_type(filename)
-        if guessed_type is not None:
-            params['ContentType'] = guessed_type
+        try:
+            guessed_type = guess_content_type(filename)
+            if guessed_type is not None:
+                params['ContentType'] = guessed_type
+        # This catches a bug in the mimetype libary where some MIME types
+        # specifically on windows machines cause a UnicodeDecodeError
+        # because the MIME type in the Windows registery has an encoding
+        # that cannot be properly encoded using the default system encoding.
+        # https://bugs.python.org/issue9291
+        #
+        # So instead of hard failing, just log the issue and fall back to the
+        # default guessed content type of None.
+        except UnicodeDecodeError:
+            LOGGER.debug(
+                'Unable to guess content type for %s due to '
+                'UnicodeDecodeError: ', filename, exc_info=True
+            )
 
     def download(self):
         """
@@ -277,6 +299,7 @@ class FileInfo(TaskInfo):
         """
         bucket, key = find_bucket_key(self.src)
         params = {'Bucket': bucket, 'Key': key}
+        RequestParamsMapper.map_get_object_params(params, self.parameters)
         response_data = self.client.get_object(**params)
         save_file(self.dest, response_data, self.last_update,
                   self.is_stream)
@@ -285,20 +308,21 @@ class FileInfo(TaskInfo):
         """
         Copies a object in s3 to another location in s3.
         """
-        copy_source = self.src
+        source_bucket, source_key = find_bucket_key(self.src)
+        copy_source = {'Bucket': source_bucket, 'Key': source_key}
         bucket, key = find_bucket_key(self.dest)
         params = {'Bucket': bucket,
                   'CopySource': copy_source, 'Key': key}
-        self._handle_object_params(params)
-        self._handle_metadata_directive(params)
-        self.client.copy_object(**params)
+        self._inject_content_type(params)
+        RequestParamsMapper.map_copy_object_params(params, self.parameters)
+        response_data = self.client.copy_object(**params)
 
     def delete(self):
         """
         Deletes the file from s3 or local.  The src file and type is used
         from the file info object.
         """
-        if (self.src_type == 's3'):
+        if self.src_type == 's3':
             bucket, key = find_bucket_key(self.src)
             params = {'Bucket': bucket, 'Key': key}
             self.source_client.delete_object(**params)
@@ -324,7 +348,9 @@ class FileInfo(TaskInfo):
     def create_multipart_upload(self):
         bucket, key = find_bucket_key(self.dest)
         params = {'Bucket': bucket, 'Key': key}
-        self._handle_object_params(params)
+        self._inject_content_type(params)
+        RequestParamsMapper.map_create_multipart_upload_params(
+            params, self.parameters)
         response_data = self.client.create_multipart_upload(**params)
         upload_id = response_data['UploadId']
         return upload_id
