@@ -13,6 +13,7 @@
 import os
 import sys
 
+from botocore.client import Config
 from dateutil.parser import parse
 from dateutil.tz import tzlocal
 
@@ -27,7 +28,8 @@ from awscli.customizations.s3.fileinfo import TaskInfo, FileInfo
 from awscli.customizations.s3.filters import create_filter
 from awscli.customizations.s3.s3handler import S3Handler, S3StreamHandler
 from awscli.customizations.s3.utils import find_bucket_key, uni_print, \
-    AppendFilter, find_dest_path_comp_key, human_readable_size
+    AppendFilter, find_dest_path_comp_key, human_readable_size, \
+    RequestParamsMapper
 from awscli.customizations.s3.syncstrategy.base import MissingFileSync, \
     SizeAndLastModifiedSync, NeverSync
 from awscli.customizations.s3 import transferconfig
@@ -63,7 +65,12 @@ QUIET = {'name': 'quiet', 'action': 'store_true',
 
 FORCE = {'name': 'force', 'action': 'store_true',
          'help_text': (
-             "Deletes all objects in the bucket including the bucket itself.")}
+             "Deletes all objects in the bucket including the bucket itself. "
+             "Note that versioned objects will not be deleted in this "
+             "process which would cause the bucket deletion to fail because "
+             "the bucket would not be empty. To delete versioned "
+             "objects use the ``s3api delete-object`` command with "
+             "the ``--version-id`` parameter.")}
 
 
 FOLLOW_SYMLINKS = {'name': 'follow-symlinks', 'action': 'store_true',
@@ -91,7 +98,7 @@ NO_GUESS_MIME_TYPE = {'name': 'no-guess-mime-type', 'action': 'store_false',
                           "file is guessed when it is uploaded.")}
 
 
-CONTENT_TYPE = {'name': 'content-type', 'nargs': 1,
+CONTENT_TYPE = {'name': 'content-type',
                 'help_text': (
                     "Specify an explicit content type for this operation.  "
                     "This value overrides any guessed mime types.")}
@@ -108,32 +115,41 @@ INCLUDE = {'name': 'include', 'action': AppendFilter, 'nargs': 1,
            'dest': 'filters',
            'help_text': (
                "Don't exclude files or objects "
-               "in the command that match the specified pattern")}
+               "in the command that match the specified pattern. "
+               'See <a href="http://docs.aws.amazon.com/cli/latest/reference'
+               '/s3/index.html#use-of-exclude-and-include-filters">Use of '
+               'Exclude and Include Filters</a> for details.')}
 
 
-ACL = {'name': 'acl', 'nargs': 1,
+ACL = {'name': 'acl',
        'choices': ['private', 'public-read', 'public-read-write',
                    'authenticated-read', 'bucket-owner-read',
                    'bucket-owner-full-control', 'log-delivery-write'],
        'help_text': (
            "Sets the ACL for the object when the command is "
-           "performed.  Only accepts values of ``private``, ``public-read``, "
+           "performed.  If you use this parameter you must have the "
+           '"s3:PutObjectAcl" permission included in the list of actions '
+           "for your IAM policy. "
+           "Only accepts values of ``private``, ``public-read``, "
            "``public-read-write``, ``authenticated-read``, "
            "``bucket-owner-read``, ``bucket-owner-full-control`` and "
-           "``log-delivery-write``.")}
+           "``log-delivery-write``. "
+           'See <a href="http://docs.aws.amazon.com/AmazonS3/latest/dev/'
+           'acl-overview.html#canned-acl">Canned ACL</a> for details')}
 
 
 GRANTS = {
     'name': 'grants', 'nargs': '+',
     'help_text': (
-        'Grant specific permissions to individual users or groups. You '
-        'can supply a list of grants of the form::<p/>  --grants '
+        '<p>Grant specific permissions to individual users or groups. You '
+        'can supply a list of grants of the form</p><codeblock>--grants '
         'Permission=Grantee_Type=Grantee_ID [Permission=Grantee_Type='
-        'Grantee_ID ...]<p/>To specify the same permission type for multiple '
-        'grantees, specify the permission as such::<p/> --grants '
+        'Grantee_ID ...]</codeblock>To specify the same permission type '
+        'for multiple '
+        'grantees, specify the permission as such as <codeblock>--grants '
         'Permission=Grantee_Type=Grantee_ID,Grantee_Type=Grantee_ID,...'
-        '</p>Each value contains the following elements:'
-        '<p/><ul><li><code>Permission</code> - Specifies '
+        '</codeblock>Each value contains the following elements:'
+        '<ul><li><code>Permission</code> - Specifies '
         'the granted permissions, and can be set to read, readacl, '
         'writeacl, or full.</li><li><code>Grantee_Type</code> - '
         'Specifies how the grantee is to be identified, and can be set '
@@ -152,20 +168,89 @@ GRANTS = {
         'UsingAuthAccess.html">Access Control</a>')}
 
 
-SSE = {'name': 'sse', 'action': 'store_true',
-       'help_text': (
-           "Enable Server Side Encryption of the object in S3")}
+SSE = {
+    'name': 'sse', 'nargs': '?', 'const': 'AES256',
+    'choices': ['AES256', 'aws:kms'],
+    'help_text': (
+        'Specifies server-side encryption of the object in S3. '
+        'Valid values are ``AES256`` and ``aws:kms``. If the parameter is '
+        'specified but no value is provided, ``AES256`` is used.'
+    )
+}
 
 
-STORAGE_CLASS = {'name': 'storage-class', 'nargs': 1,
-                 'choices': ['STANDARD', 'REDUCED_REDUNDANCY'],
+SSE_C = {
+    'name': 'sse-c', 'nargs': '?', 'const': 'AES256', 'choices': ['AES256'],
+    'help_text': (
+        'Specifies server-side encryption using customer provided keys '
+        'of the the object in S3. ``AES256`` is the only valid value. '
+        'If the parameter is specified but no value is provided, '
+        '``AES256`` is used. If you provide this value, ``--sse-c-key`` '
+        'be specfied as well.'
+    )
+}
+
+
+SSE_C_KEY = {
+    'name': 'sse-c-key',
+    'help_text': (
+        'The customer-provided encryption key to use to server-side '
+        'encrypt the object in S3. If you provide this value, '
+        '``--sse-c`` be specfied as well.'
+    )
+}
+
+
+SSE_KMS_KEY_ID = {
+    'name': 'sse-kms-key-id',
+    'help_text': (
+        'The AWS KMS key ID that should be used to server-side '
+        'encrypt the object in S3. Note that you should only '
+        'provide this parameter if KMS key ID is different the '
+        'default S3 master KMS key.'
+    )
+}
+
+
+SSE_C_COPY_SOURCE = {
+    'name': 'sse-c-copy-source', 'nargs': '?',
+    'const': 'AES256', 'choices': ['AES256'],
+    'help_text': (
+        'This parameter should only be specified when copying an S3 object '
+        'that was encrypted server-side with a customer-provided '
+        'key. It specifies the algorithm to use when decrypting the source '
+        'object. ``AES256`` is the only valid '
+        'value. If the parameter is specified but no value is provided, '
+        '``AES256`` is used. If you provide this value, '
+        '``--sse-c-copy-source-key`` be specfied as well. '
+    )
+}
+
+
+SSE_C_COPY_SOURCE_KEY = {
+    'name': 'sse-c-copy-source-key',
+    'help_text': (
+        'This parameter should only be specified when copying an S3 object '
+        'that was encrypted server-side with a customer-provided '
+        'key. Specifies the customer-provided encryption key for Amazon S3 '
+        'to use to decrypt the source object. The encryption key provided '
+        'must be one that was used when the source object was created. '
+        'If you provide this value, ``--sse-c-copy-source`` be specfied as '
+        'well.'
+    )
+}
+
+
+STORAGE_CLASS = {'name': 'storage-class',
+                 'choices': ['STANDARD', 'REDUCED_REDUNDANCY', 'STANDARD_IA'],
                  'help_text': (
                      "The type of storage to use for the object. "
-                     "Valid choices are: STANDARD | REDUCED_REDUNDANCY. "
+                     "Valid choices are: STANDARD | REDUCED_REDUNDANCY "
+                     "| STANDARD_IA. "
                      "Defaults to 'STANDARD'")}
 
 
-WEBSITE_REDIRECT = {'name': 'website-redirect', 'nargs': 1,
+WEBSITE_REDIRECT = {'name': 'website-redirect',
                     'help_text': (
                         "If the bucket is configured as a website, "
                         "redirects requests for this object to another object "
@@ -174,19 +259,19 @@ WEBSITE_REDIRECT = {'name': 'website-redirect', 'nargs': 1,
                         "metadata.")}
 
 
-CACHE_CONTROL = {'name': 'cache-control', 'nargs': 1,
+CACHE_CONTROL = {'name': 'cache-control',
                  'help_text': (
                      "Specifies caching behavior along the "
                      "request/reply chain.")}
 
 
-CONTENT_DISPOSITION = {'name': 'content-disposition', 'nargs': 1,
+CONTENT_DISPOSITION = {'name': 'content-disposition',
                        'help_text': (
                            "Specifies presentational information "
                            "for the object.")}
 
 
-CONTENT_ENCODING = {'name': 'content-encoding', 'nargs': 1,
+CONTENT_ENCODING = {'name': 'content-encoding',
                     'help_text': (
                         "Specifies what content encodings have been "
                         "applied to the object and thus what decoding "
@@ -194,11 +279,11 @@ CONTENT_ENCODING = {'name': 'content-encoding', 'nargs': 1,
                         "referenced by the Content-Type header field.")}
 
 
-CONTENT_LANGUAGE = {'name': 'content-language', 'nargs': 1,
+CONTENT_LANGUAGE = {'name': 'content-language',
                     'help_text': ("The language the content is in.")}
 
 
-SOURCE_REGION = {'name': 'source-region', 'nargs': 1,
+SOURCE_REGION = {'name': 'source-region',
                  'help_text': (
                      "When transferring objects from an s3 bucket to an s3 "
                      "bucket, this specifies the region of the source bucket."
@@ -210,14 +295,31 @@ SOURCE_REGION = {'name': 'source-region', 'nargs': 1,
 
 
 EXPIRES = {
-    'name': 'expires', 'nargs': 1,
+    'name': 'expires',
     'help_text': (
         "The date and time at which the object is no longer cacheable.")
 }
 
 
+METADATA = {
+    'name': 'metadata', 'cli_type_name': 'map',
+    'schema': {
+        'type': 'map',
+        'key': {'type': 'string'},
+        'value': {'type': 'string'}
+    },
+    'help_text': (
+        "A map of metadata to store with the objects in S3. This will be "
+        "applied to every object which is part of this request. In a sync, this"
+        "means that files which haven't changed won't receive the new metadata."
+        " When copying between two s3 locations, the metadata-directive "
+        "argument will default to 'REPLACE' unless otherwise specified."
+    )
+}
+
+
 METADATA_DIRECTIVE = {
-    'name': 'metadata-directive', 'nargs': 1, 'choices': ['COPY', 'REPLACE'],
+    'name': 'metadata-directive', 'choices': ['COPY', 'REPLACE'],
     'help_text': (
         'Specifies whether the metadata is copied from the source object '
         'or replaced with metadata provided when copying S3 objects. '
@@ -278,17 +380,31 @@ PAGE_SIZE = {'name': 'page-size', 'cli_type_name': 'integer',
                  'Using a lower value may help if an operation times out.')}
 
 
+IGNORE_GLACIER_WARNINGS = {
+    'name': 'ignore-glacier-warnings', 'action': 'store_true',
+    'help_text': (
+        'Turns off glacier warnings. Warnings about operations that cannot '
+        'be performed because it involves copying, downloading, or moving '
+        'a glacier object will no longer be printed to standard error and '
+        'will no longer cause the return code of the command to be ``2``.'
+    )
+}
+
+
 TRANSFER_ARGS = [DRYRUN, QUIET, INCLUDE, EXCLUDE, ACL,
                  FOLLOW_SYMLINKS, NO_FOLLOW_SYMLINKS, NO_GUESS_MIME_TYPE,
-                 SSE, STORAGE_CLASS, GRANTS, WEBSITE_REDIRECT, CONTENT_TYPE,
-                 CACHE_CONTROL, CONTENT_DISPOSITION, CONTENT_ENCODING,
-                 CONTENT_LANGUAGE, EXPIRES, SOURCE_REGION, ONLY_SHOW_ERRORS,
-                 PAGE_SIZE]
+                 SSE, SSE_C, SSE_C_KEY, SSE_KMS_KEY_ID, SSE_C_COPY_SOURCE,
+                 SSE_C_COPY_SOURCE_KEY, STORAGE_CLASS, GRANTS,
+                 WEBSITE_REDIRECT, CONTENT_TYPE, CACHE_CONTROL,
+                 CONTENT_DISPOSITION, CONTENT_ENCODING, CONTENT_LANGUAGE,
+                 EXPIRES, SOURCE_REGION, ONLY_SHOW_ERRORS,
+                 PAGE_SIZE, IGNORE_GLACIER_WARNINGS]
 
 
-def get_client(session, region, endpoint_url, verify):
+def get_client(session, region, endpoint_url, verify, config=None):
     return session.create_client('s3', region_name=region,
-                                 endpoint_url=endpoint_url, verify=verify)
+                                 endpoint_url=endpoint_url, verify=verify,
+                                 config=config)
 
 
 class S3Command(BasicCommand):
@@ -303,7 +419,7 @@ class ListCommand(S3Command):
     DESCRIPTION = ("List S3 objects and common prefixes under a prefix or "
                    "all S3 buckets. Note that the --output argument "
                    "is ignored for this command.")
-    USAGE = "<S3Path> or NONE"
+    USAGE = "<S3Uri> or NONE"
     ARG_TABLE = [{'name': 'paths', 'nargs': '?', 'default': 's3://',
                   'positional_arg': True, 'synopsis': USAGE}, RECURSIVE,
                  PAGE_SIZE, HUMAN_READABLE, SUMMARIZE]
@@ -442,7 +558,7 @@ class ListCommand(S3Command):
 class WebsiteCommand(S3Command):
     NAME = 'website'
     DESCRIPTION = 'Set the website configuration for a bucket.'
-    USAGE = '<S3Path>'
+    USAGE = '<S3Uri>'
     ARG_TABLE = [{'name': 'paths', 'nargs': 1, 'positional_arg': True,
                   'synopsis': USAGE}, INDEX_DOCUMENT, ERROR_DOCUMENT]
 
@@ -491,6 +607,7 @@ class S3TransferCommand(S3Command):
         cmd_params.add_page_size(parsed_args)
         cmd_params.add_paths(parsed_args.paths)
         self._handle_rm_force(parsed_globals, cmd_params.parameters)
+
         runtime_config = transferconfig.RuntimeConfig().build_config(
             **self._session.get_scoped_config().get('s3', {}))
         cmd = CommandArchitecture(self._session, self.NAME,
@@ -522,8 +639,9 @@ class S3TransferCommand(S3Command):
 
     def _handle_rm_force(self, parsed_globals, parameters):
         """
-        This function recursive deletes objects in a bucket if the force
-        parameters was thrown when using the remove bucket command.
+        This function recursively deletes objects in a bucket if the force
+        parameter was thrown when using the remove bucket command. It will
+        refuse to delete if a key is specified in the s3path.
         """
         # XXX: This shouldn't really be here.  This was originally moved from
         # the CommandParameters class to here, but this is still not the ideal
@@ -535,7 +653,10 @@ class S3TransferCommand(S3Command):
         # to be kept simple.
         if 'force' in parameters:
             if parameters['force']:
-                bucket = find_bucket_key(parameters['src'][5:])[0]
+                bucket, key = find_bucket_key(parameters['src'][5:])
+                if key:
+                    raise ValueError('Please specify a valid bucket name only.'
+                                     ' E.g. s3://%s' % bucket)
                 path = 's3://' + bucket
                 del_objects = RmCommand(self._session)
                 del_objects([path, '--recursive'], parsed_globals)
@@ -545,27 +666,27 @@ class CpCommand(S3TransferCommand):
     NAME = 'cp'
     DESCRIPTION = "Copies a local file or S3 object to another location " \
                   "locally or in S3."
-    USAGE = "<LocalPath> <S3Path> or <S3Path> <LocalPath> " \
-            "or <S3Path> <S3Path>"
+    USAGE = "<LocalPath> <S3Uri> or <S3Uri> <LocalPath> " \
+            "or <S3Uri> <S3Uri>"
     ARG_TABLE = [{'name': 'paths', 'nargs': 2, 'positional_arg': True,
                   'synopsis': USAGE}] + TRANSFER_ARGS + \
-                [METADATA_DIRECTIVE, EXPECTED_SIZE, RECURSIVE]
+                [METADATA, METADATA_DIRECTIVE, EXPECTED_SIZE, RECURSIVE]
 
 
 class MvCommand(S3TransferCommand):
     NAME = 'mv'
     DESCRIPTION = "Moves a local file or S3 object to " \
                   "another location locally or in S3."
-    USAGE = "<LocalPath> <S3Path> or <S3Path> <LocalPath> " \
-            "or <S3Path> <S3Path>"
+    USAGE = "<LocalPath> <S3Uri> or <S3Uri> <LocalPath> " \
+            "or <S3Uri> <S3Uri>"
     ARG_TABLE = [{'name': 'paths', 'nargs': 2, 'positional_arg': True,
-                  'synopsis': USAGE}] + TRANSFER_ARGS + [METADATA_DIRECTIVE,
-                                                         RECURSIVE]
+                  'synopsis': USAGE}] + TRANSFER_ARGS +\
+                [METADATA, METADATA_DIRECTIVE, RECURSIVE]
 
 class RmCommand(S3TransferCommand):
     NAME = 'rm'
     DESCRIPTION = "Deletes an S3 object."
-    USAGE = "<S3Path>"
+    USAGE = "<S3Uri>"
     ARG_TABLE = [{'name': 'paths', 'nargs': 1, 'positional_arg': True,
                   'synopsis': USAGE}, DRYRUN, QUIET, RECURSIVE, INCLUDE,
                  EXCLUDE, ONLY_SHOW_ERRORS, PAGE_SIZE]
@@ -577,24 +698,31 @@ class SyncCommand(S3TransferCommand):
                   "new and updated files from the source directory to " \
                   "the destination. Only creates folders in the destination " \
                   "if they contain one or more files."
-    USAGE = "<LocalPath> <S3Path> or <S3Path> " \
-            "<LocalPath> or <S3Path> <S3Path>"
+    USAGE = "<LocalPath> <S3Uri> or <S3Uri> " \
+            "<LocalPath> or <S3Uri> <S3Uri>"
     ARG_TABLE = [{'name': 'paths', 'nargs': 2, 'positional_arg': True,
-                  'synopsis': USAGE}] + TRANSFER_ARGS
+                  'synopsis': USAGE}] + TRANSFER_ARGS + \
+                [METADATA, METADATA_DIRECTIVE]
 
 
 class MbCommand(S3TransferCommand):
     NAME = 'mb'
     DESCRIPTION = "Creates an S3 bucket."
-    USAGE = "<S3Path>"
+    USAGE = "<S3Uri>"
     ARG_TABLE = [{'name': 'paths', 'nargs': 1, 'positional_arg': True,
                   'synopsis': USAGE}]
 
 
 class RbCommand(S3TransferCommand):
     NAME = 'rb'
-    DESCRIPTION = "Deletes an S3 bucket."
-    USAGE = "<S3Path>"
+    DESCRIPTION = (
+        "Deletes an empty S3 bucket. A bucket must be completely empty "
+        "of objects and versioned objects before it can be deleted. "
+        "However, the ``--force`` parameter can be used to delete "
+        "the non-versioned objects in the bucket before the bucket is "
+        "deleted."
+    )
+    USAGE = "<S3Uri>"
     ARG_TABLE = [{'name': 'paths', 'nargs': 1, 'positional_arg': True,
                   'synopsis': USAGE}, FORCE]
 
@@ -621,25 +749,31 @@ class CommandArchitecture(object):
         self._source_client = None
 
     def set_clients(self):
+        client_config = None
+        if self.parameters.get('sse') == 'aws:kms':
+            client_config = Config(signature_version='s3v4')
         self._client = get_client(
             self.session,
             region=self.parameters['region'],
             endpoint_url=self.parameters['endpoint_url'],
-            verify=self.parameters['verify_ssl']
+            verify=self.parameters['verify_ssl'],
+            config=client_config
         )
         self._source_client = get_client(
             self.session,
             region=self.parameters['region'],
             endpoint_url=self.parameters['endpoint_url'],
-            verify=self.parameters['verify_ssl']
+            verify=self.parameters['verify_ssl'],
+            config=client_config
         )
         if self.parameters['source_region']:
             if self.parameters['paths_type'] == 's3s3':
                 self._source_client = get_client(
                     self.session,
-                    region=self.parameters['source_region'][0],
+                    region=self.parameters['source_region'],
                     endpoint_url=None,
-                    verify=self.parameters['verify_ssl']
+                    verify=self.parameters['verify_ssl'],
+                    config=client_config
                 )
 
     def create_instructions(self):
@@ -733,15 +867,43 @@ class CommandArchitecture(object):
         }
         result_queue = queue.Queue()
         operation_name = cmd_translation[paths_type][self.cmd]
-        file_generator = FileGenerator(self._source_client,
-                                       operation_name,
-                                       self.parameters['follow_symlinks'],
-                                       self.parameters['page_size'],
-                                       result_queue=result_queue)
-        rev_generator = FileGenerator(self._client, '',
-                                      self.parameters['follow_symlinks'],
-                                      self.parameters['page_size'],
-                                      result_queue=result_queue)
+
+        fgen_kwargs = {
+            'client': self._source_client, 'operation_name': operation_name,
+            'follow_symlinks': self.parameters['follow_symlinks'],
+            'page_size': self.parameters['page_size'],
+            'result_queue': result_queue
+        }
+        rgen_kwargs = {
+            'client': self._client, 'operation_name': '',
+            'follow_symlinks': self.parameters['follow_symlinks'],
+            'page_size': self.parameters['page_size'],
+            'result_queue': result_queue
+        }
+
+        fgen_request_parameters = {}
+        fgen_head_object_params = {}
+        fgen_request_parameters['HeadObject'] = fgen_head_object_params
+        fgen_kwargs['request_parameters'] = fgen_request_parameters
+
+        # SSE-C may be neaded for HeadObject for copies/downloads/deletes
+        # If the operation is s3 to s3, the FileGenerator should use the
+        # copy source key and algorithm. Otherwise, use the regular
+        # SSE-C key and algorithm. Note the reverse FileGenerator does
+        # not need any of these because it is used only for sync operations
+        # which only use ListObjects which does not require HeadObject.
+        RequestParamsMapper.map_head_object_params(
+            fgen_head_object_params, self.parameters)
+        if paths_type == 's3s3':
+            RequestParamsMapper.map_head_object_params(
+                fgen_head_object_params, {
+                    'sse_c': self.parameters.get('sse_c_copy_source'),
+                    'sse_c_key': self.parameters.get('sse_c_copy_source_key')
+                }
+            )
+
+        file_generator = FileGenerator(**fgen_kwargs)
+        rev_generator = FileGenerator(**rgen_kwargs)
         taskinfo = [TaskInfo(src=files['src']['path'],
                              src_type='s3',
                              operation_name=operation_name,
@@ -876,6 +1038,7 @@ class CommandParameters(object):
             self.parameters['dest'] = paths[0]
         self._validate_streaming_paths()
         self._validate_path_args()
+        self._validate_sse_c_args()
 
     def _validate_streaming_paths(self):
         self.parameters['is_stream'] = False
@@ -894,6 +1057,20 @@ class CommandParameters(object):
             raise ValueError("Cannot mv a file onto itself: '%s' - '%s'" % (
                 params['src'], params['dest']))
 
+        # If the user provided local path does not exist, hard fail because
+        # we know that we will not be able to upload the file.
+        if 'locals3' == params['paths_type'] and not params['is_stream']:
+            if not os.path.exists(params['src']):
+                raise RuntimeError(
+                    'The user-provided path %s does not exist.' %
+                    params['src'])
+        # If the operation is downloading to a directory that does not exist,
+        # create the directories so no warnings are thrown during the syncing
+        # process.
+        elif 's3local' == params['paths_type'] and params['dir_op']:
+            if not os.path.exists(params['dest']):
+                os.makedirs(params['dest'])
+
     def _same_path(self, src, dest):
         if not self.parameters['paths_type'] == 's3s3':
             return False
@@ -937,35 +1114,6 @@ class CommandParameters(object):
         else:
             raise TypeError("%s\nError: Invalid argument type" % usage)
 
-    def check_src_path(self, paths):
-        """
-        This checks the source paths to deem if they are valid.  The check
-        performed in S3 is first it lists the objects using the source path.
-        If there is an error like the bucket does not exist, the error will be
-        caught with ``check_error()`` function.  If the operation is on a
-        single object in s3, it checks that a list of object was returned and
-        that the first object listed is the name of the specified in the
-        command line.  If the operation is on objects under a common prefix,
-        it will check that there are common prefixes and objects under
-        the specified prefix.
-        For local files, it first checks that the path exists.  Then it checks
-        that the path is a directory if it is a directory operation or that
-        the path is a file if the operation is on a single file.
-        """
-        src_path = paths[0]
-        dir_op = self.parameters['dir_op']
-        if not src_path.startswith('s3://'):
-            src_path = os.path.abspath(src_path)
-            if os.path.exists(src_path):
-                if os.path.isdir(src_path) and not dir_op:
-                    raise Exception("Error: Requires a local file")
-                elif os.path.isfile(src_path) and dir_op:
-                    raise Exception("Error: Requires a local directory")
-                else:
-                    pass
-            else:
-                raise Exception("Error: Local path does not exist")
-
     def add_region(self, parsed_globals):
         self.parameters['region'] = parsed_globals.region
 
@@ -984,3 +1132,33 @@ class CommandParameters(object):
 
     def add_page_size(self, parsed_args):
         self.parameters['page_size'] = getattr(parsed_args, 'page_size', None)
+
+    def _validate_sse_c_args(self):
+        self._validate_sse_c_arg()
+        self._validate_sse_c_arg('sse_c_copy_source')
+        self._validate_sse_c_copy_source_for_paths()
+
+    def _validate_sse_c_arg(self, sse_c_type='sse_c'):
+        sse_c_key_type = sse_c_type + '_key'
+        sse_c_type_param = '--' + sse_c_type.replace('_', '-')
+        sse_c_key_type_param = '--' + sse_c_key_type.replace('_', '-')
+        if self.parameters.get(sse_c_type):
+            if not self.parameters.get(sse_c_key_type):
+                raise ValueError(
+                    'It %s is specified, %s must be specified '
+                    'as well.' % (sse_c_type_param, sse_c_key_type_param)
+                )
+        if self.parameters.get(sse_c_key_type):
+            if not self.parameters.get(sse_c_type):
+                raise ValueError(
+                    'It %s is specified, %s must be specified '
+                    'as well.' % (sse_c_key_type_param, sse_c_type_param)
+                )
+
+    def _validate_sse_c_copy_source_for_paths(self):
+        if self.parameters.get('sse_c_copy_source'):
+            if self.parameters['paths_type'] != 's3s3':
+                raise ValueError(
+                    '--sse-c-copy-source is only supported for '
+                    'copy operations.'
+                )
