@@ -11,7 +11,8 @@ from botocore.vendored.requests.packages.urllib3.exceptions import \
     ReadTimeoutError
 
 from awscli.customizations.s3.utils import find_bucket_key, MD5Error, \
-    ReadFileChunk, relative_path, IORequest, IOCloseRequest, PrintTask
+    ReadFileChunk, relative_path, IORequest, IOCloseRequest, PrintTask, \
+    RequestParamsMapper
 
 
 LOGGER = logging.getLogger(__name__)
@@ -126,12 +127,13 @@ class BasicTask(OrderableTask):
 
 class CopyPartTask(OrderableTask):
     def __init__(self, part_number, chunk_size,
-                 result_queue, upload_context, filename):
+                 result_queue, upload_context, filename, params):
         self._result_queue = result_queue
         self._upload_context = upload_context
         self._part_number = part_number
         self._chunk_size = chunk_size
         self._filename = filename
+        self._params = params
 
     def _is_last_part(self, part_number):
         return self._part_number == int(
@@ -159,8 +161,10 @@ class CopyPartTask(OrderableTask):
             params = {'Bucket': bucket, 'Key': key,
                       'PartNumber': self._part_number,
                       'UploadId': upload_id,
-                      'CopySource': '%s/%s' % (src_bucket, src_key),
+                      'CopySource': {'Bucket': src_bucket, 'Key': src_key},
                       'CopySourceRange': range_param}
+            RequestParamsMapper.map_upload_part_copy_params(
+                params, self._params)
             response_data = self._filename.client.upload_part_copy(**params)
             etag = response_data['CopyPartResult']['ETag'][1:-1]
             self._upload_context.announce_finished_part(
@@ -199,12 +203,13 @@ class UploadPartTask(OrderableTask):
     object.
     """
     def __init__(self, part_number, chunk_size, result_queue, upload_context,
-                 filename, payload=None):
+                 filename, params, payload=None):
         self._result_queue = result_queue
         self._upload_context = upload_context
         self._part_number = part_number
         self._chunk_size = chunk_size
         self._filename = filename
+        self._params = params
         self._payload = payload
 
     def _read_part(self):
@@ -231,6 +236,7 @@ class UploadPartTask(OrderableTask):
                       'PartNumber': self._part_number,
                       'UploadId': upload_id,
                       'Body': body}
+            RequestParamsMapper.map_upload_part_params(params, self._params)
             try:
                 response_data = self._filename.client.upload_part(**params)
             finally:
@@ -263,9 +269,10 @@ class UploadPartTask(OrderableTask):
 
 
 class CreateLocalFileTask(OrderableTask):
-    def __init__(self, context, filename):
+    def __init__(self, context, filename, result_queue):
         self._context = context
         self._filename = filename
+        self._result_queue = result_queue
 
     def __call__(self):
         dirname = os.path.dirname(self._filename.dest)
@@ -284,6 +291,11 @@ class CreateLocalFileTask(OrderableTask):
             with open(self._filename.dest, 'wb'):
                 pass
         except Exception as e:
+            message = print_operation(self._filename, failed=True,
+                                      dryrun=False)
+            message += '\n' + str(e)
+            result = {'message': message, 'error': True}
+            self._result_queue.put(PrintTask(**result))
             self._context.cancel()
         else:
             self._context.announce_file_created()
@@ -329,7 +341,7 @@ class DownloadPartTask(OrderableTask):
     TOTAL_ATTEMPTS = 5
 
     def __init__(self, part_number, chunk_size, result_queue,
-                 filename, context, io_queue):
+                 filename, context, io_queue, params):
         self._part_number = part_number
         self._chunk_size = chunk_size
         self._result_queue = result_queue
@@ -337,6 +349,7 @@ class DownloadPartTask(OrderableTask):
         self._client = filename.client
         self._context = context
         self._io_queue = io_queue
+        self._params = params
 
     def __call__(self):
         try:
@@ -359,7 +372,10 @@ class DownloadPartTask(OrderableTask):
         LOGGER.debug("Downloading bytes range of %s for file %s", range_param,
                      self._filename.dest)
         bucket, key = find_bucket_key(self._filename.src)
-        params = {'Bucket': bucket, 'Key': key, 'Range': range_param}
+        params = {'Bucket': bucket,
+                  'Key': key,
+                  'Range': range_param}
+        RequestParamsMapper.map_get_object_params(params, self._params)
         for i in range(self.TOTAL_ATTEMPTS):
             try:
                 LOGGER.debug("Making GetObject requests with byte range: %s",
@@ -494,7 +510,8 @@ class CompleteMultipartUploadTask(BasicTask):
             'MultipartUpload': {'Parts': parts},
         }
         try:
-            self.filename.client.complete_multipart_upload(**params)
+            response_data = self.filename.client.complete_multipart_upload(
+                **params)
         except Exception as e:
             LOGGER.debug("Error trying to complete multipart upload: %s",
                          e, exc_info=True)
