@@ -81,6 +81,22 @@ INTEG_LOG = logging.getLogger('awscli.tests.integration')
 AWS_CMD = None
 
 
+def skip_if_windows(reason):
+    """Decorator to skip tests that should not be run on windows.
+
+    Example usage:
+
+        @skip_if_windows("Not valid")
+        def test_some_non_windows_stuff(self):
+            self.assertEqual(...)
+
+    """
+    def decorator(func):
+        return unittest.skipIf(
+            platform.system() not in ['Darwin', 'Linux'], reason)(func)
+    return decorator
+
+
 def create_clidriver():
     driver = awscli.clidriver.create_clidriver()
     session = driver.session
@@ -163,7 +179,19 @@ def create_bucket(session, name=None, region=None):
     params = {'Bucket': bucket_name}
     if region != 'us-east-1':
         params['CreateBucketConfiguration'] = {'LocationConstraint': region}
-    client.create_bucket(**params)
+    try:
+        # To disable the (obsolete) awscli.errorhandler.ClientError behavior
+        client.meta.events.unregister(
+            'after-call', unique_id='awscli-error-handler')
+        client.create_bucket(**params)
+    except ClientError as e:
+        if e.response['Error'].get('Code') == 'BucketAlreadyOwnedByYou':
+            # This can happen in the retried request, when the first one
+            # succeeded on S3 but somehow the response never comes back.
+            # We still got a bucket ready for test anyway.
+            pass
+        else:
+            raise
     return bucket_name
 
 
@@ -301,6 +329,7 @@ class BaseAWSCommandParamsTest(unittest.TestCase):
         self.environ_patch.stop()
         if self.make_request_is_patched:
             self.make_request_patch.stop()
+            self.make_request_is_patched = False
 
     def before_call(self, params, **kwargs):
         self._store_params(params)
@@ -310,6 +339,13 @@ class BaseAWSCommandParamsTest(unittest.TestCase):
         self.last_params = params['body']
 
     def patch_make_request(self):
+        # If you do not stop a previously started patch,
+        # it can never be stopped if you call start() again on the same
+        # patch again...
+        # So stop the current patch before calling start() on it again.
+        if self.make_request_is_patched:
+            self.make_request_patch.stop()
+            self.make_request_is_patched = False
         make_request_patch = self.make_request_patch.start()
         if self.parsed_responses is not None:
             make_request_patch.side_effect = lambda *args, **kwargs: \
